"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var Configuration_1 = require("./Configuration");
var index_1 = require("./util/index");
var tsioc_1 = require("tsioc");
var path = require("path");
var core_1 = require("./core");
var child_process_1 = require("child_process");
var logs = require("./logs/index");
var index_2 = require("./aop/index");
/**
 * Bootstrap
 *
 * @export
 * @class Bootstrap
 */
var Bootstrap = /** @class */ (function () {
    /**
     * Creates an instance of WebApplication.
     * @param {string} rootdir
     * @param {Type<any>} [appType]
     * @memberof WebApplication
     */
    function Bootstrap(rootdir, appType) {
        this.rootdir = rootdir;
        this.appType = appType;
        this.middlewares = [];
        this.appType = this.appType || core_1.Application;
    }
    /**
     * create new application.
     *
     * @static
     * @param {string} rootdir
     * @param {Type<Application>} [appType]
     * @returns
     * @memberof WebApplication
     */
    Bootstrap.create = function (rootdir, appType) {
        return new Bootstrap(rootdir, appType);
    };
    /**
     * user custom IContainer.
     *
     * @param {(IContainer | Promise<IContainer>)} [container]
     * @returns
     *
     * @memberOf WebHostBuilder
     */
    Bootstrap.prototype.useContainer = function (container) {
        if (container) {
            if (!this.container) {
                this.container = index_1.Defer.create();
            }
            this.container.resolve(container);
        }
        return this;
    };
    Bootstrap.prototype.getContainer = function () {
        if (!this.container) {
            this.useContainer(this.createContainer());
        }
        return this.container.promise;
    };
    Bootstrap.prototype.createContainer = function (option) {
        return this.getContainerBuilder().build(option);
    };
    Bootstrap.prototype.useContainerBuilder = function (builder) {
        this.builder = builder;
        return this;
    };
    Bootstrap.prototype.getContainerBuilder = function () {
        if (!this.builder) {
            this.builder = new tsioc_1.ContainerBuilder();
        }
        return this.builder;
    };
    /**
     * use custom configuration.
     *
     * @param {(string | IConfiguration)} [config]
     * @returns {Bootstrap}
     *
     * @memberOf WebHostBuilder
     */
    Bootstrap.prototype.useConfiguration = function (config) {
        if (!this.configDefer) {
            this.configDefer = index_1.Defer.create();
            this.configDefer.resolve(new Configuration_1.Configuration());
        }
        var cfgmodeles;
        if (tsioc_1.isString(config)) {
            if (fs_1.existsSync(config)) {
                cfgmodeles = require(config);
            }
            else if (child_process_1.execFileSync(path.join(this.rootdir, config))) {
                cfgmodeles = require(path.join(this.rootdir, config));
            }
            else {
                console.log("config file: " + config + " not exists.");
            }
        }
        else if (config) {
            cfgmodeles = config;
        }
        else {
            var cfgpath_1 = path.join(this.rootdir, './config');
            ['.js', '.ts', '.json'].forEach(function (ext) {
                if (cfgmodeles) {
                    return false;
                }
                if (fs_1.existsSync(cfgpath_1 + ext)) {
                    cfgmodeles = require(cfgpath_1 + ext);
                    return false;
                }
                return true;
            });
            if (!cfgmodeles) {
                console.log('your app has not config file.');
            }
        }
        if (cfgmodeles) {
            var excfg_1 = (cfgmodeles['default'] ? cfgmodeles['default'] : cfgmodeles);
            this.configDefer.promise = this.configDefer.promise
                .then(function (cfg) {
                cfg = Object.assign(cfg || {}, excfg_1 || {});
                return cfg;
            });
        }
        return this;
    };
    Bootstrap.prototype.getConfiguration = function () {
        if (!this.configDefer) {
            this.useConfiguration();
        }
        return this.configDefer.promise;
    };
    /**
     * use middleware `fn` or  `MiddlewareFactory`.
     * @param {MvcMiddleware} middleware
     * @returns {Bootstrap}
     * @memberOf WebHostBuilder
     */
    Bootstrap.prototype.use = function (middleware) {
        this.middlewares.push(middleware);
        return this;
    };
    /**
     * run service.
     *
     * @param {Function} [listener]
     * @returns
     * @memberof Bootstrap
     */
    Bootstrap.prototype.run = function (listener) {
        return __awaiter(this, void 0, void 0, function () {
            var app, config, server, port;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.build()];
                    case 1:
                        app = _a.sent();
                        config = app.container.get(Configuration_1.Configuration);
                        server = app.getServer();
                        port = config.port || parseInt(process.env.PORT || '0');
                        if (config.hostname) {
                            server.listen(port, config.hostname, listener);
                        }
                        else {
                            server.listen(port, listener);
                        }
                        console.log('service listen on port: ', port);
                        return [2 /*return*/, app];
                }
            });
        });
    };
    /**
     * build application.
     * @returns {Bootstrap}
     * @memberOf WebHostBuilder
     */
    Bootstrap.prototype.build = function () {
        return __awaiter(this, void 0, void 0, function () {
            var cfg, container, app;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getConfiguration()];
                    case 1:
                        cfg = _a.sent();
                        return [4 /*yield*/, this.getContainer()];
                    case 2:
                        container = _a.sent();
                        return [4 /*yield*/, this.initIContainer(cfg, container)];
                    case 3:
                        _a.sent();
                        app = container.get(this.appType);
                        this.setupMiddwares(app, container, cfg.beforeMiddlewares, cfg.excludeMiddlewares);
                        this.setupMiddwares(app, container, cfg.useMiddlewares, cfg.excludeMiddlewares.concat(cfg.afterMiddlewares));
                        this.setupRoutes(cfg, container);
                        this.setupMiddwares(app, container, cfg.afterMiddlewares, cfg.excludeMiddlewares);
                        return [2 /*return*/, app];
                }
            });
        });
    };
    Bootstrap.prototype.initIContainer = function (config, container) {
        return __awaiter(this, void 0, void 0, function () {
            var modules, controllers, aops;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        config.rootdir = config.rootdir ? tsioc_1.toAbsolutePath(this.rootdir, config.rootdir) : this.rootdir;
                        container.registerSingleton(Configuration_1.Configuration, config);
                        container.registerSingleton(index_1.mvcSymbols.IConfiguration, config);
                        this.registerDefaults(container);
                        // register app.
                        container.register(this.appType);
                        if (!this.middlewares) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                modules: this.middlewares.filter(function (m) { return tsioc_1.isClass(m); })
                            })];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!config.middlewares) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.middlewares
                            })];
                    case 3:
                        modules = _a.sent();
                        if (!config.useMiddlewares || config.useMiddlewares.length < 1) {
                            config.useMiddlewares = this.middlewares.concat(modules);
                        }
                        else {
                            config.useMiddlewares = this.middlewares.concat(config.useMiddlewares);
                        }
                        _a.label = 4;
                    case 4:
                        // register default
                        this.registerDefaultMiddlewars(container);
                        if (!config.controllers) return [3 /*break*/, 6];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.controllers
                            })];
                    case 5:
                        controllers = _a.sent();
                        if (!config.useControllers || config.useControllers.length < 1) {
                            config.useControllers = controllers;
                        }
                        _a.label = 6;
                    case 6:
                        this.builder.snycLoadModule(container, { modules: [logs] });
                        container.register(index_2.AnnotationLogerAspect);
                        if (config.debug) {
                            container.register(index_2.DebugAspect);
                        }
                        container.register(index_2.AuthAspect);
                        if (!config.aop) return [3 /*break*/, 8];
                        return [4 /*yield*/, this.builder.loadModule(container, {
                                basePath: config.rootdir,
                                files: config.aop
                            })];
                    case 7:
                        aops = _a.sent();
                        config.usedAops = aops;
                        _a.label = 8;
                    case 8: return [2 /*return*/, container];
                }
            });
        });
    };
    Bootstrap.prototype.registerDefaultMiddlewars = function (container) {
        core_1.registerDefaultMiddlewars(container);
    };
    Bootstrap.prototype.registerDefaults = function (container) {
        core_1.registerDefaults(container);
    };
    Bootstrap.prototype.setupRoutes = function (config, container) {
        var router = container.get(config.routerMiddlewate || core_1.Router);
        router.register.apply(router, config.useControllers);
        router.setup();
    };
    Bootstrap.prototype.setupMiddwares = function (app, container, middlewares, excludes) {
        middlewares.forEach(function (m) {
            if (!m) {
                return;
            }
            if (excludes && excludes.length > 0 && excludes.indexOf(m) > 0) {
                return;
            }
            if (tsioc_1.isClass(m) || tsioc_1.isString(m) || tsioc_1.isSymbol(m)) {
                var middleware = container.get(m);
                if (tsioc_1.isFunction(middleware.setup)) {
                    middleware.setup();
                }
            }
            else if (tsioc_1.isFunction(m)) {
                app.use(m);
            }
        });
        return app;
    };
    return Bootstrap;
}());
exports.Bootstrap = Bootstrap;

//# sourceMappingURL=sourcemaps/Bootstrap.js.map
