"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tsioc_1 = require("tsioc");
/**
 * request method.
 */
var RequestMethod;
(function (RequestMethod) {
    RequestMethod[RequestMethod["Get"] = 0] = "Get";
    RequestMethod[RequestMethod["Post"] = 1] = "Post";
    RequestMethod[RequestMethod["Put"] = 2] = "Put";
    RequestMethod[RequestMethod["Delete"] = 3] = "Delete";
    RequestMethod[RequestMethod["Options"] = 4] = "Options";
    RequestMethod[RequestMethod["Head"] = 5] = "Head";
    RequestMethod[RequestMethod["Patch"] = 6] = "Patch";
})(RequestMethod = exports.RequestMethod || (exports.RequestMethod = {}));
/**
 * convert method to string.
 *
 * @export
 * @param {RequestMethod} [method]
 * @returns {string}
 */
function methodToString(method) {
    if (tsioc_1.isString(method)) {
        return method.toUpperCase();
    }
    switch (method) {
        case RequestMethod.Get:
            return 'GET';
        case RequestMethod.Post:
            return 'POST';
        case RequestMethod.Put:
            return 'PUT';
        case RequestMethod.Delete:
            return 'DELETE';
        case RequestMethod.Options:
            return 'OPTIONS';
        case RequestMethod.Head:
            return 'HEAD';
        case RequestMethod.Patch:
            return 'PATCH';
        default:
            return '';
    }
}
exports.methodToString = methodToString;
/**
 * parse string to RequestMethod
 *
 * @export
 * @param {string} [method]
 * @returns {RequestMethod}
 */
function parseRequestMethod(method) {
    method = (method || '').toUpperCase();
    switch (method) {
        case 'GET':
            return RequestMethod.Get;
        case 'POST':
            return RequestMethod.Post;
        case 'PUT':
            return RequestMethod.Put;
        case 'DELETE':
            return RequestMethod.Delete;
        case 'OPTIONS':
            return RequestMethod.Options;
        case 'HEAD':
            return RequestMethod.Head;
        case 'PATCH':
            return RequestMethod.Patch;
        default:
            return null;
    }
}
exports.parseRequestMethod = parseRequestMethod;

//# sourceMappingURL=../sourcemaps/core/RequestMethod.js.map
